@@ -2,8 +2,10 @@
 ##' 
 ##' Probability density, distribution, quantile, random generation, hazard, 
 ##' cumulative hazard, mean and restricted mean functions for the Royston/Parmar
-##' spline model.   These functions have all parameters of the distribution collecte together in a single argument \code{gamma}.  For the equivalent functions with one argument per parameter, see \code{\link{Survsplinek}}. 
-##' 
+##' spline model. These functions have all parameters of the distribution collected
+##' together in a single argument \code{gamma}. For the equivalent functions with
+##' one argument per parameter, see \code{\link{Survsplinek}}.
+##'
 ##' @aliases dsurvspline psurvspline qsurvspline rsurvspline
 ##' hsurvspline Hsurvspline mean_survspline rmst_survspline
 ##' 
@@ -86,7 +88,6 @@ NULL
 
 ## Things to do that are common to d/p/q functions
 ## could be generalized to any function with vector of arguments 
-## TODO put in h,H functions
 ## TODO more special value handling
 
 dbase.survspline <- function(q, gamma, knots, scale, offset=0, deriv=FALSE){
@@ -97,11 +98,11 @@ dbase.survspline <- function(q, gamma, knots, scale, offset=0, deriv=FALSE){
     q <- rep(q, length=nret)
     offset <- rep(offset, length=nret)
 
-    gamma <- apply(gamma, 2, rep, length=nret)
-    knots <- apply(knots, 2, rep, length=nret)
-    
-    if(!is.matrix(gamma)) gamma <- matrix(gamma, nrow=nret)
-    if(!is.matrix(knots)) knots <- matrix(knots, nrow=nret)
+    gamma <- matrix(rep(as.numeric(t(gamma)), length.out = ncol(gamma) * nret),
+                    ncol = ncol(gamma), byrow = TRUE)
+    knots <- matrix(rep(as.numeric(t(knots)), length.out = ncol(knots) * nret),
+                    ncol = ncol(knots), byrow = TRUE)
+
     if (ncol(gamma) != ncol(knots)) {
         stop("length of gamma should equal number of knots")
     }
@@ -267,6 +268,7 @@ hsurvspline <- function(x, gamma, beta=0, X=0, knots=c(-10,10), scale="hazard",
         eta <- rowSums(basis(knots, tsfn(q,timescale)) * gamma) + as.numeric(X %*% beta) + offset
         eeta <- hlink(scale)(eta)
         ret[ind] <- dtsfn(q, timescale) * rowSums(dbasis(knots, tsfn(q, timescale)) * gamma) * eeta
+        ret[ind][ret[ind]<=0] <- 0 # these correspond to invalid decreasing cumulative hazard functions
     }
     as.numeric(ret)
 }
@@ -274,13 +276,22 @@ hsurvspline <- function(x, gamma, beta=0, X=0, knots=c(-10,10), scale="hazard",
 ##' @rdname Survspline
 ##' @export
 rmst_survspline = function(t, gamma, beta=0, X=0, knots=c(-10,10), scale="hazard", timescale="log", offset=0, start=0){
-  rmst_generic(psurvspline, t, start=start, gamma=gamma, knots=knots, beta=beta, X=X, scale=scale, timescale=timescale, offset=offset)
+    rmst_generic(psurvspline, t, start=start,
+                 matargs = c("gamma", "knots"), scalarargs = c("scale", "timescale"), 
+                 gamma=gamma, knots=knots,
+                 beta=beta, X=X,
+                 scale=scale, timescale=timescale, offset=offset)
 }
 
 ##' @rdname Survspline
 ##' @export
 mean_survspline = function(gamma, beta=0, X=0, knots=c(-10,10), scale="hazard", timescale="log", offset=0){
-  rmst_generic(psurvspline, Inf, start=0, gamma=gamma, knots=knots, beta=beta, X=X, scale=scale, timescale=timescale, offset=offset)
+    nt <- if (is.matrix(gamma)) nrow(gamma) else 1
+    rmst_generic(psurvspline, rep(Inf,nt), start=0,
+                 matargs = c("gamma", "knots"), 
+                 scalarargs = c("scale", "timescale"), 
+                 gamma=gamma, knots=knots,
+                 beta=beta, X=X, scale=scale, timescale=timescale, offset=offset)
 }
 
 ##' Natural cubic spline basis
@@ -765,7 +776,9 @@ flexsurvspline <- function(formula, data, weights, bhazard, rtrunc, subset,
     args$bhazard <- temp$bhazard
     args$rtrunc <- temp$rtrunc
     args$subset <- temp$subset
-    if (is.infinite(do.call("flexsurvreg", args)$loglik)){
+    lik_try <- do.call("flexsurvreg", args)$loglik
+    inits_fail <- is.infinite(lik_try) || is.na(lik_try)
+    if (inits_fail){
         args$dist$inits <- flexsurv.splineinits.cox
     }
     args$fixedpars <- fpold
